sbindir="/usr/sbin"
bindir="/usr/bin"
LIBEXEC="/usr/lib/xen"
LIBEXEC_BIN="/usr/lib/xen/bin"
libdir="/usr/lib"
SHAREDIR="/usr/share"
XENFIRMWAREDIR="/usr/lib/xen/boot"
XEN_CONFIG_DIR="/etc/xen"
XEN_SCRIPT_DIR="/etc/xen/scripts"
XEN_LOCK_DIR="/var/lock"
XEN_RUN_DIR="/var/run/xen"
XEN_PAGING_DIR="/var/lib/xen/xenpaging"
XEN_DUMP_DIR="/var/lib/xen/dump"
XEN_LOG_DIR="/var/log/xen"
XEN_LIB_DIR="/var/lib/xen"
XEN_RUN_STORED="/var/run/xenstored"
